#include "CivInpDbHelper.h"
#include "CivTableFields.h"
#include "CivConnection.h"
#include "CivPgDbConnection.h"


CivInpDbHelper::CivInpDbHelper(const std::string& uri)
	:CivInpHelperAbs(uri)
{
	mCondtion = "1=1 ";
}

CivInpDbHelper::~CivInpDbHelper()
{
}

bool CivInpDbHelper::getNode(CivNode& node)
{
	JunctionTable nodeTableFields;
	std::vector<std::string> fields;

	fields.push_back(nodeTableFields.sn);
	fields.push_back(nodeTableFields.elev);
	fields.push_back(nodeTableFields.baseDemand);
	fields.push_back(nodeTableFields.demandPattern);

	std::vector<std::map<std::string,std::string>> resultVector;	
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);

	// ༭װ
	size_t total = resultVector.size();
	for (int i = 0; i < total; i++)
	{
		std::map<std::string, std::string> map = resultVector[i];
		CivNode::NodeTable nodeTable;

		nodeTable.ID = map.find(nodeTableFields.sn)->second;
		nodeTable.Elev = map.find(nodeTableFields.elev)->second;
		nodeTable.Demand = map.find(nodeTableFields.baseDemand)->second;
		nodeTable.PatternId = map.find(nodeTableFields.demandPattern)->second;

		node.addItem(nodeTable);
	}
	return true;
}

bool CivInpDbHelper::getPipe(CivPipe& pipes)
{
	PipeTable pipeTable;

	// עȥ
	// 뷧Ĺܶ
	std::string sql = "select pipe.\"\",pipe.\"ʼڵ\",pipe.\"ֹڵ\", \
            pipe.\"ܳ\", pipe.\"ܾ\",pipe.\"Ħϵ\",pipe.\"ϵ\",pipe.\"ʼ״̬\",va.\"\", pipe.\"code\",\
		    va.\"\" from \"ܶ\" as pipe, \"\" as va where  pipe.\"ʼڵ\" = va.\"\" \
            or pipe.\"ֹڵ\" = va.\"\" order by \"\"";

	if (!mDbConn->execSql(sql))
	{	
		return false;
	}

	std::vector<std::map<std::string, std::string>> resultMap;
	mDbConn->queryResult(resultMap);

	int toal = resultMap.size();
	std::string repeatSn;
	std::string repeatVaType;
	std::string starNode;
	std::string endNode;

	// 洢ƴӵĹܶCODEΪȥ
	std::set<std::string> filterSet;

	for (int i = 0; i < toal; i++)
	{
		std::map<std::string, std::string> tempMap = resultMap[i];	
		std::string  sn = tempMap.find("")->second;	
		std::string vaType = tempMap.find("")->second;

		std::string pipeCode = tempMap.find(pipeTable.code)->second;
		filterSet.insert(pipeCode);


		if (sn == repeatSn && repeatVaType == vaType && repeatVaType == "BV")
		{
			CivPipe::PipesTable pipe;
			pipe.ID = tempMap.find(pipeTable.snNo)->second;

			
			std::string secondStartNode = tempMap.find(pipeTable.startPoint)->second;
			std::string secondEndNode = tempMap.find(pipeTable.endPoint)->second;

			if (starNode == repeatSn)
			{
				pipe.Node1 = secondStartNode;
				pipe.Node2 = endNode;
			}
			else if(endNode == repeatSn)
			{
				pipe.Node1 = starNode;
				pipe.Node2 = secondEndNode;
			}
			
			pipe.Length = tempMap.find(pipeTable.length)->second;
			pipe.Diameter = tempMap.find(pipeTable.diameter)->second;
			pipe.Roughness = tempMap.find(pipeTable.friction)->second;
			pipe.MinorLoss = tempMap.find(pipeTable.localLoss)->second;
			pipe.Status = tempMap.find(pipeTable.status)->second;

			pipes.addItem(pipe);
		}
		repeatVaType = vaType;
		repeatSn = sn;
		starNode = tempMap.find(pipeTable.startPoint)->second;
		endNode = tempMap.find(pipeTable.endPoint)->second;

	}

	std::vector<std::string> fields;
	fields.push_back(pipeTable.snNo); 
	fields.push_back(pipeTable.startPoint);
	fields.push_back(pipeTable.endPoint); 
	fields.push_back(pipeTable.length); 
	fields.push_back(pipeTable.diameter); 
	fields.push_back(pipeTable.friction); 
	fields.push_back(pipeTable.localLoss);
	fields.push_back(pipeTable.status);
	fields.push_back(pipeTable.code);
	

	std::vector<std::map<std::string, std::string>> resultVector;
	std::string condition = " \"code\" not in (";
	for (auto iter = filterSet.begin(); iter != filterSet.end(); iter++)
	{
		std::string code =  *iter;
		condition.append("'"+code+"'");
		condition.append(",");
	}
	condition = condition.substr(0, condition.length() - 1);
	condition.append(")");

	mDbConn->query(PIPELINE, fields, resultVector, condition);


	size_t total = resultVector.size();
    for (int i = 0; i < total; i++)
    {
        CivPipe::PipesTable pipe;

		std::map<std::string, std::string> map = resultVector[i];

		pipe.ID = map.find(pipeTable.snNo)->second;
		pipe.Node1 = map.find(pipeTable.startPoint)->second;
		pipe.Node2 = map.find(pipeTable.endPoint)->second;
		pipe.Length = map.find(pipeTable.length)->second;
		pipe.Diameter = map.find(pipeTable.diameter)->second;
		pipe.Roughness = map.find(pipeTable.friction)->second;
		pipe.MinorLoss = map.find(pipeTable.localLoss)->second;
		pipe.Status = map.find(pipeTable.status)->second;
		pipe.code = map.find(pipeTable.code)->second;
		pipes.addItem(pipe);
    }
	return true;
}

bool CivInpDbHelper::getTank(CivTank& tanks)
{
	TankTable tankTable;

	std::vector<std::string> fields;

	fields.push_back(tankTable.sn);
	fields.push_back(tankTable.elev);
	fields.push_back(tankTable.initLevel);
	fields.push_back(tankTable.minLevel);
	fields.push_back(tankTable.maxLevel);
	fields.push_back(tankTable.diametor);
	fields.push_back(tankTable.minVolume);
	fields.push_back(tankTable.volumeCurve);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(TANK, fields, resultVector, mCondtion);

	size_t total = resultVector.size();
	for (int i = 0; i < total; i++)
	{
		CivTank::TankTable tank;
		std::map<std::string, std::string>map = resultVector[i];

		tank.ID = map.find(tankTable.sn)->second;
		tank.Elev = map.find(tankTable.elev)->second;
		tank.InitLevel = map.find(tankTable.initLevel)->second;
		tank.MinLevel = map.find(tankTable.minLevel)->second;
		tank.MaxLevel = map.find(tankTable.maxLevel)->second;
		tank.Diameter = map.find(tankTable.diametor)->second;
		tank.MinVol = map.find(tankTable.minVolume)->second;
		tank.VolCurve = map.find(tankTable.volumeCurve)->second;
		
		tanks.addItem(tank);
	}
	return true;
}

bool CivInpDbHelper::getValve(CivValve& valves)
{
	ValveTable vaveTable;
	std::vector<std::string> fields;

	fields.push_back(vaveTable.sn); 
	fields.push_back(vaveTable.startPoint);
	fields.push_back(vaveTable.endPoint);
	fields.push_back(vaveTable.length);
	fields.push_back(vaveTable.type);
	fields.push_back(vaveTable.setting);
	fields.push_back(vaveTable.lossCoeff);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(VALVE, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivValve::ValveTable vave;
		std::map<std::string, std::string> map = resultVector[i];

		std::string vaType = map.find(vaveTable.type)->second;
		if (vaType == "BV")
			continue;

		vave.ID = map.find(vaveTable.sn)->second;
		vave.Node1 = map.find(vaveTable.startPoint)->second;
		vave.Node2 = map.find(vaveTable.endPoint)->second;
		vave.Diameter = map.find(vaveTable.length)->second;
		vave.Type = map.find(vaveTable.type)->second;
		vave.Setting = map.find(vaveTable.setting)->second;
		vave.MinorLoss = map.find(vaveTable.lossCoeff)->second;

		valves.addItem(vave);
	}
	return true;
}

bool CivInpDbHelper::getPumps(CivPumps& pumps)
{
	PumpTable pmTable;
	std::vector<std::string> fields;

	fields.push_back(pmTable.sn);
	fields.push_back(pmTable.startPoint);
	fields.push_back(pmTable.endPoint);
	fields.push_back(pmTable.headCurve);
	fields.push_back(pmTable.power);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PUMP, fields, resultVector, mCondtion);
	
	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivPumps::PumpTable pump;
		std::map<std::string, std::string> map = resultVector[i];

		pump.ID = map.find(pmTable.sn)->second;
		pump.Node1 = map.find(pmTable.startPoint)->second;
		pump.Node2 = map.find(pmTable.endPoint)->second;
		std::string param;
		
		std::string curve = map.find(pmTable.headCurve)->second;
		if (curve != "")
			param.append("head " + curve);

		std::string power = map.find(pmTable.power)->second;
		if (power != "")
			param.append(" power " + power);
	
		pump.Parameters = param;
		pumps.addItem(pump);
	}
	return true;
}

bool CivInpDbHelper::getReservoirs(CivReservoirs& reservoirs)
{
	ResourcesTable restemTable;

	std::vector<std::string> fields;

	fields.push_back(restemTable.sn);
	fields.push_back(restemTable.totalHead);
	fields.push_back(restemTable.headPattern);

	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(RESIVOIR, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivReservoirs::ReservoirsTable resTable;
		std::map<std::string, std::string> map = resultVector[i];

		resTable.ID = map.find(restemTable.sn)->second;
		resTable.Head = map.find(restemTable.totalHead)->second;
		resTable.Pattern = map.find(restemTable.headPattern)->second;

		reservoirs.addItem(resTable);
	}
	return true;
}

bool CivInpDbHelper::getCoordinates(CivCoordinates& coord)
{
	JunctionTable nodeTable;

	std::vector<std::string> fields;

	fields.push_back(nodeTable.sn);
	fields.push_back(nodeTable.xCoord);
	fields.push_back(nodeTable.yCoord);
	fields.push_back(nodeTable.code);


	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);
	mDbConn->query(TANK, fields, resultVector, mCondtion);
	mDbConn->query(RESIVOIR, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivCoordinates::CoordTable coordTable;
		std::map<std::string, std::string> map = resultVector[i];

		coordTable.ID = map.find(fields[0])->second;
		coordTable.XCoord = map.find(fields[1])->second;
		coordTable.YCoord = map.find(fields[2])->second;
		coordTable.code = map.find(nodeTable.code)->second;

		coord.addItem(coordTable);
	}

	return true;
}

bool CivInpDbHelper::getQuality(CivQuality& quality)
{
	JunctionTable nodeTable;

	std::vector<std::string> fields;

	fields.push_back(nodeTable.sn);
	fields.push_back(nodeTable.initQuality);
	fields.push_back(nodeTable.chlorine);


	std::vector<std::map<std::string, std::string>> resultVector;
	mDbConn->query(PIPENODE, fields, resultVector, mCondtion);
	mDbConn->query(RESIVOIR, fields, resultVector,mCondtion);
	mDbConn->query(TANK, fields, resultVector, mCondtion);

	size_t totals = resultVector.size();
	for (int i = 0; i < totals; i++)
	{
		CivQuality::QualityTable quliTable;

		std::map<std::string, std::string> map = resultVector[i];
		quliTable.ID = map.find(nodeTable.sn)->second;
		switch (mQualityType)
		{
		
		case Chlorine:
			quliTable.InitQuality = map.find(nodeTable.chlorine)->second;
			break;
		case Age:
		default:
			quliTable.InitQuality = map.find(nodeTable.initQuality)->second;
			break;
		}

		quality.addItem(quliTable);
	}

	return true;
}